<?php
namespace bwdsiax_ScrollingIconAnimation_namespace\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class bwdsiax_ScrollingIconAnimation_widgets extends Widget_Base {

	public function get_name() {
		return esc_html__( 'bwdsiax-scrolling-icon-animation', 'bwdsiax-icon-animation' );
	}

	public function get_title() {
		return esc_html__( 'Trigger Lottie Animation', 'bwdsiax-icon-animation' );
	}

	public function get_icon() {
		return 'bwdsiax-effective-icon';
	}

	public function get_categories() {
		return [ 'bwdsiax-scrolling-icon-animation-category' ];
	}

	protected function register_controls() {

		$this->start_controls_section(
			'bwdsiax_section_general_settings',
			array(
				'label' => __( 'General Settings', 'bwdsiax-icon-animation' ),
			)
		);

		$this->add_control(
			'bwdsiax_source',
			array(
				'label'   => __( 'File Source', 'bwdsiax-icon-animation' ),
				'type'    => Controls_Manager::SELECT,
				'options' => array(
					'url'  => __( 'External URL', 'bwdsiax-icon-animation' ),
					'file' => __( 'Media File', 'bwdsiax-icon-animation' ),
				),
				'default' => 'url',
			)
		);

		$this->add_control(
			'bwdsiax_lottie_url',
			array(
				'label'       => __( 'Animation JSON URL', 'bwdsiax-icon-animation' ),
				'type'        => Controls_Manager::TEXT,
				'dynamic'     => array( 'active' => true ),
				'default' 		=> 'https://assets5.lottiefiles.com/packages/lf20_1vK4588Atx.json',
				'description' => 'Get URL/JSON file from <a href="https://lottiefiles.com/featured" target="_blank">here...</a>',
				'label_block' => true,
				'condition'   => array(
					'bwdsiax_source' => 'url',
				),
			)
		);

		$this->add_control(
			'bwdsiax_lottie_file',
			array(
				'label'              => __( 'Upload JSON File', 'bwdsiax-icon-animation' ),
				'type'               => Controls_Manager::MEDIA,
				'media_type'         => 'application/json',
				'frontend_available' => true,
				'condition'          => array(
					'bwdsiax_source' => 'file',
				),
			)
		);

		$this->add_control(
			'bwdsiax_lottie_loop',
			array(
				'label'        => __( 'Loop', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::SWITCHER,
				'return_value' => 'true',
				'default'      => 'true',
			)
		);

		$this->add_control(
			'bwdsiax_lottie_reverse',
			array(
				'label'        => __( 'Reverse', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::SWITCHER,
				'return_value' => 'true',
			)
		);

		$this->add_control(
			'bwdsiax_lottie_speed',
			array(
				'label'   => __( 'Animation Speed', 'bwdsiax-icon-animation' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 1,
				'min'     => 0.1,
				'max'     => 100,
				'step'    => 0.1,
			)
		);

		$this->add_control(
			'bwdsiax_trigger',
			array(
				'label' => __( 'Trigger', 'bwdsiax-icon-animation' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'scroll',
				'options' => array(
					'none' => __( 'None', 'bwdsiax-icon-animation' ),
					'hover' => __( 'Hover', 'bwdsiax-icon-animation' ),
					'scroll' => __( 'Scroll', 'bwdsiax-icon-animation' ),
				),
				'frontend_available' => true,
			)
		);

		$this->add_control(
			'bwdsiax_hover_leave_reverse',
			array(
				'label'        => __( 'Reset on Mouse Leave', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::SWITCHER,
				'return_value' => 'true',
				'condition'    => array(
					'bwdsiax_trigger'         => 'hover',
					'lottie_reverse!' => 'true',
				),
			)
		);

		$this->add_control(
			'bwdsiax_lottie_hover',
			array(
				'label'        => __( 'Play on Hover', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::HIDDEN,
				'return_value' => 'true',
			)
		);

		$this->add_control(
			'bwdsiax_animate_on_scroll',
			array(
				'label'        => __( 'Play On Scroll', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::HIDDEN,
				'return_value' => 'true',
			)
		);

		$this->add_control(
			'bwdsiax_animate_speed',
			array(
				'label'     => __( 'Speed', 'bwdsiax-icon-animation' ),
				'type'      => Controls_Manager::SLIDER,
				'default'   => array(
					'size' => 4,
				),
				'range'     => array(
					'px' => array(
						'max'  => 10,
						'step' => 0.1,
					),
				),
				'condition' => array(
					'bwdsiax_trigger'         => 'scroll',
					'lottie_reverse!' => 'true',
				),
			)
		);

		$this->add_control(
			'bwdsiax_animate_view',
			array(
				'label'     => __( 'Viewport', 'bwdsiax-icon-animation' ),
				'type'      => Controls_Manager::SLIDER,
				'default'   => array(
					'sizes' => array(
						'start' => 0,
						'end'   => 100,
					),
					'unit'  => '%',
				),
				'labels'    => array(
					__( 'Bottom', 'bwdsiax-icon-animation' ),
					__( 'Top', 'bwdsiax-icon-animation' ),
				),
				'scales'    => 1,
				'handles'   => 'range',
				'condition' => array(
					'bwdsiax_trigger'         => array( 'scroll', 'viewport' ),
					'lottie_reverse!' => 'true',
				),
			)
		);

		$this->add_responsive_control(
			'bwdsiax_animation_size',
			array(
				'label'      => __( 'Size', 'bwdsiax-icon-animation' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => array( 'px', 'em', '%' ),
				'default'    => array(
					'unit' => 'px',
					'size' => 200,
				),
				'range'      => array(
					'px' => array(
						'min' => 1,
						'max' => 800,
					),
					'em' => array(
						'min' => 1,
						'max' => 30,
					),
				),
				'separator'  => 'before',
				'selectors'  => array(
					'{{WRAPPER}}.bwdsiax-lottie-canvas .bwdsiax-lottie-animation, {{WRAPPER}}.bwdsiax-lottie-svg svg'    => 'width: {{SIZE}}{{UNIT}} !important;',
				),
			)
		);

		$this->add_responsive_control(
			'bwdsiax_lottie_rotate',
			array(
				'label'       => __( 'Rotate (degrees)', 'bwdsiax-icon-animation' ),
				'type'        => Controls_Manager::SLIDER,
				'description' => __( 'Set rotation value in degrees', 'bwdsiax-icon-animation' ),
				'range'       => array(
					'px' => array(
						'min' => -180,
						'max' => 180,
					),
				),
				'default'     => array(
					'size' => 0,
				),
				'selectors'   => array(
					'{{WRAPPER}} .bwdsiax-lottie-animation' => 'transform: rotate({{SIZE}}deg)',
				),
			)
		);

		$this->add_responsive_control(
			'bwdsiax_animation_align',
			array(
				'label'     => __( 'Alignment', 'bwdsiax-icon-animation' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => array(
					'left'   => array(
						'title' => __( 'Left', 'bwdsiax-icon-animation' ),
						'icon'  => 'eicon-text-align-left',
					),
					'center' => array(
						'title' => __( 'Center', 'bwdsiax-icon-animation' ),
						'icon'  => 'eicon-text-align-center',
					),
					'right'  => array(
						'title' => __( 'Right', 'bwdsiax-icon-animation' ),
						'icon'  => 'eicon-text-align-right',
					),
				),
				'default'   => 'center',
				'toggle'    => false,
				'separator' => 'before',
				'selectors' => array(
					'{{WRAPPER}} .elementor-widget-container' => 'text-align: {{VALUE}}',
				),
			)
		);

		$this->add_control(
			'bwdsiax_link_switcher',
			array(
				'label' => __( 'Link', 'bwdsiax-icon-animation' ),
				'type'  => Controls_Manager::SWITCHER,
			)
		);
		$this->add_control(
			'link',
			array(
				'label'       => __( 'Place URL', 'bwdsiax-icon-animation' ),
				'type'        => Controls_Manager::URL,
				'dynamic'     => array( 'active' => true ),
				'default'     => array(
					'url' => '#',
				),
				'placeholder' => 'https://example.com/',
				'label_block' => true,
				'separator'   => 'after',
				'condition'   => array(
					'bwdsiax_link_switcher'  => 'yes',
				),
			)
		);
		$this->add_control(
			'bwdsiax_lottie_renderer',
			array(
				'label'        => __( 'Render As', 'bwdsiax-icon-animation' ),
				'type'         => Controls_Manager::SELECT,
				'options'      => array(
					'svg'    => __( 'SVG', 'bwdsiax-icon-animation' ),
					'canvas' => __( 'Canvas', 'bwdsiax-icon-animation' ),
				),
				'default'      => 'svg',
				'prefix_class' => 'bwdsiax-lottie-',
				'render_type'  => 'template',
				'label_block'  => true,
				'separator'    => 'before',
			)
		);
		$this->end_controls_section();

		$this->start_controls_section(
			'bwdsiax_catpost_carousel_box_style',
			[
				'label' => esc_html__( 'Wrap Style', 'bwdsiax-icon-animation' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);
		// Hover control start for box
		$this->start_controls_tabs(
			'bwdsiax_box_style_post'
		);
		$this->start_controls_tab(
			'bwdsiax_box_normal_post',
			[
				'label' => esc_html__( 'Normal', 'bwdsiax-icon-animation' ),
			]
		);
		// Normal Controls
		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsiax_box_bg_grediant_color',
				'label' => esc_html__( 'Background', 'bwdsiax-icon-animation' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation svg',
			]
		);
		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsiax_box_boxshadow',
				'label' => esc_html__( 'Box Shadow', 'bwdsiax-icon-animation' ),
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation svg',
			]
		);
		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsiax_box_border',
				'label' => esc_html__( 'Border', 'bwdsiax-icon-animation' ),
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation svg',
			]
		);
		$this->end_controls_tab();
		$this->start_controls_tab(
			'bwdsiax_box_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'bwdsiax-icon-animation' ),
			]
		);
		// Hover Controls
		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsiax_box_bg_grediant_color_hover',
				'label' => esc_html__( 'Background', 'bwdsiax-icon-animation' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation:hover svg',
			]
		);
		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsiax_box_boxshadow_hover',
				'label' => esc_html__( 'Box Shadow', 'bwdsiax-icon-animation' ),
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation:hover svg',
			]
		);
		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsiax_box_border_hover',
				'label' => esc_html__( 'Border', 'bwdsiax-icon-animation' ),
				'selector' => '{{WRAPPER}} .bwdsiax-lottie-animation:hover svg',
			]
		);
		$this->end_controls_tab();
		$this->end_controls_tabs();
		// Hover Control End

		$this->add_responsive_control(
			'bwdsiax_the_box_border_radius',
			[
				'label' => esc_html__('Border Radius', 'bwdsiax-icon-animation'),
				'type' => Controls_Manager::DIMENSIONS,
				'separator' => 'before',
				'size_units' => ['px', 'em', 'rem', '%'],
				'selectors' => [
					'{{WRAPPER}} .bwdsiax-lottie-animation svg' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			'bwdsiax_the_box_border_bottom',
			[
				'label' => esc_html__('Margin', 'bwdsiax-icon-animation'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'rem', '%'],
				'selectors' => [
					'{{WRAPPER}} .bwdsiax-lottie-animation svg' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			'bwdsiax_box_padding',
			[
				'label' => esc_html__('Padding', 'bwdsiax-icon-animation'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'rem', '%'],
				'selectors' => [
					'{{WRAPPER}} .bwdsiax-lottie-animation svg' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->end_controls_section();
	}

	protected function render() {
		$settings = $this->get_settings_for_display();
		if(!empty($settings['bwdsiax_lottie_url'] || $settings['bwdsiax_lottie_file']['url'])){
			$anim_url = 'url' === $settings['bwdsiax_source'] ? $settings['bwdsiax_lottie_url'] : $settings['bwdsiax_lottie_file']['url'];
			if ( empty( $anim_url ) ) {
				return;
			}
			if ( '' !== $settings['bwdsiax_trigger'] ) {
				$settings['bwdsiax_lottie_hover'] = false;
				$settings['bwdsiax_animate_on_scroll'] = false;
			}
			$this->add_render_attribute(
				'lottie',
				array(
					'class' => 'bwdsiax-lottie-animation',
					'data-lottie-url' => $anim_url,
					'data-lottie-loop' => $settings['bwdsiax_lottie_loop'],
					'data-lottie-reverse' => $settings['bwdsiax_lottie_reverse'],
					'data-lottie-hover' => $settings['bwdsiax_lottie_hover'] || 'hover' === $settings['bwdsiax_trigger'],
					'data-lottie-speed' => $settings['bwdsiax_lottie_speed'],
					'data-lottie-render' => $settings['bwdsiax_lottie_renderer'],
				)
			);
			if ( 'hover' === $settings['bwdsiax_trigger'] && 'true' === $settings['bwdsiax_hover_leave_reverse'] ) {
				$this->add_render_attribute( 'lottie', 'data-lottie-reset', 'true' );
			}
			if ( $settings['bwdsiax_animate_on_scroll'] || 'scroll' === $settings['bwdsiax_trigger'] ) {
				$this->add_render_attribute(
					'lottie',
					array(
						'class' => 'bwdsiax-lottie-scroll',
						'data-lottie-scroll' => 'true',
						'data-scroll-start' => isset( $settings['bwdsiax_animate_view']['sizes']['start'] ) ? $settings['bwdsiax_animate_view']['sizes']['start'] : '0',
						'data-scroll-end' => isset( $settings['bwdsiax_animate_view']['sizes']['end'] ) ? $settings['bwdsiax_animate_view']['sizes']['end'] : '100',
						'data-scroll-speed' => $settings['bwdsiax_animate_speed']['size'],
					)
				);
			} elseif( 'viewport' === $settings['bwdsiax_trigger']){
				$this->add_render_attribute(
					'lottie',
					array(
						'data-lottie-viewport' => 'true', 'data-scroll-start' => $settings['bwdsiax_animate_view']['sizes']['start'], 'data-scroll-end' => $settings['bwdsiax_animate_view']['sizes']['end'],
					)
				);
			}
			if ( 'yes' === $settings['bwdsiax_link_switcher'] ) {
					$this->add_link_attributes( 'link', $settings['link'] );
			}
			echo 'yes' === $settings['bwdsiax_link_switcher']?'<a href="'.$settings['link']['url'].'">':'';
				echo '<div '.wp_kses_post( $this->get_render_attribute_string( 'lottie' ) ).'></div>';
			echo 'yes' === $settings['bwdsiax_link_switcher']?'</a>':'';
		}else{echo esc_html__('Add the Lottie link/file.');}
	}
}